#!/usr/local/bin/perl

use strict;

package GetTruth;
use Exporter;
our (@ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw(isBogon parseIOS parseJUNOS parse getFromGrnoc writeSets @routers %routeraddrs %addrs);

require LWP::UserAgent;
use HTTP::Request::Common;
require HTML::TreeBuilder;

our @routers;
our %routeraddrs;
our %addrs;

sub isBogon($) {
    $_ = shift;
    return
	/^0\./ ||
	/^127\./ ||
	/^169\.254\./ ||
	/^192\.0\.2\./ ||
	/^198\.1[89]\./ ||
	/^2(2[4-9]|3[0-9])\.\./ ||
	/^255\./ ||
	/^10\./ ||
	/^172\.(1[6-9]|2[0-9]|3[01])\./ ||
	/^192\.168\./;
}

sub getFromGrnoc($$) {
    my ($name, $debug) = @_;

    my $url = "http://routerproxy.grnoc.iu.edu/$name/";
    my $ua;
    my $response;
    my @devices;

    if (!$debug) {
	$ua = LWP::UserAgent->new;

	$response = $ua->request(GET $url);
	if (!$response->is_success) {
	    die "error reading device list: " . $response->status_line . "\n";
	}

	my $tree = HTML::TreeBuilder->new; # empty tree
	$tree->parse_content($response->content);

	# find first <div class="devices">
	# (there may be more than one, but only the first contains IP routers)
	my $element = $tree->look_down(_tag => "div", class => "devices");
	# find every <input name="device"> in the div
	my @inputs;
	if ($element) {
	    @inputs = $element->look_down(_tag => "input", name => "device");
	}

	for my $input (@inputs) {
	    push @devices, $input->attr('value');
	}
    } else {
	my $ls = `/bin/ls -1 device.*`;
	@devices = map { s/device\.//; $_; } split(/\n/, $ls);
    }

    print STDERR "# devices:  ", join(' ', @devices), "\n";

DEVICE:
    for my $device (@devices) {
	my $text;
	if (!$debug) {
	    print STDERR "# getting info for $device...\n";
	    my $query = "${url}index.cgi?fname=getResponse&args=show%20interface&cmd=show%20interface&args=&args=&args=${device}&device=${device}";
	    do {
		# sleep 1; # server will reject query if we're too fast
		select(undef, undef, undef, 0.25); # server will reject query if we're too fast
		$response = $ua->get($query);
		if (!$response->is_success) {
		    print STDERR "ERROR querying device $device: " . $response->status_line . "\n";
		    next DEVICE;
		}
	    } while ($response->content =~ /please.wait/i);

	    open RAW, ">", "device.$device";
	    print RAW $response->content;
	    close RAW;
	    $text = $response->content;

	} else {
	    $text = `cat device.$device`;
	}

	$text =~ s/&nbsp;/ /g;
#       <tr class="menu-title"><td>Response From Atlanta CRS router</td></tr>
	if (!($text =~ /Response From ([^<]*)/i)) {
	    print STDERR "warning: can't find router name for device.$device\n";
	    next;
	}
	my $router = $1;
	push @routers, $router;
	print STDERR "### router: $router\n";
	parse($router, $text);
    }

    writeSets();
}

sub parseIOS($$) {
    my ($router, $text) = @_;
    my $n = 0;
    for my $line (split("[\r\n]", $text)) {
#       Internet address is 216.24.191.224/32
	if ($line =~ /Internet\s+address\s+is\s+(\d+\.\d+\.\d+\.\d+)/i) {
	    my $addr = $1;
	    if (!isBogon($addr)) {
		$addrs{$addr}++;
		push @{$routeraddrs{$router}}, $addr;
		$n++;
	    }
	}
    }
    print STDERR "# found $n good interface addresses on $router\n"
}

sub parseJUNOS($$) {
    my ($router, $text) = @_;
    my $type;
    my $n = 0;
    for my $line (split("[\r\n]", $text)) {
	if ($line =~ /Physical interface: /) {
	    $type = undef;

	} elsif ($line =~ /Type: ([^,]*)/) {
	    $type = $1;

	} elsif ($line =~ /Local:\s+(\d+\.\d+\.\d+\.\d+)/i) {
	    my $addr = $1;
	    next if ($type =~ /software.pseudo|loopback/i);
	    if (!isBogon($addr)) {
		$addrs{$addr}++;
		push @{$routeraddrs{$router}}, $addr;
		$n++;
	    }
	}
    }
    print STDERR "# found $n good interface addresses on $router\n"
}

sub parse($$) {
    my ($router, $text) = @_;
    if ($text =~ /Physical interface: /) {
	parseJUNOS($router, $text);
    } elsif ($text =~ /Internet\s+address\s+is\s+/i) {
	parseIOS($router, $text);
    } else {
	print STDERR "warning: unknown format for $router\n";
    }
}

sub writeSets() {
    for my $addr (keys %addrs) {
	if ($addrs{$addr} > 1) {
	    print STDERR "warning: $addr found on multiple routers\n";
	}
    }

    open SETS, ">", "sets" or die "Error writing sets: $!\n";
    for my $router (sort @routers) {
	print SETS "# name: $router\n";
	print SETS join(' ', sort grep {$addrs{$_} == 1} @{$routeraddrs{$router}}), "\n";
    }
    close SETS;
}

1;
